//+------------------------------------------------------------------+
//|                                              T.O.P.S.(DSVT1).mq4 |
//|                                Trend Oriented Positioning System |
//|                                                  dsvt1@yahoo.com |
//|------------------------------------------------------------------|
//| Revision history                                                 |
//| #       Date     Author                                          |                              !
//|------------------------------------------------------------------|
//|                                                                  |
//| 1.0.1   11.12.05 isotonic                                        |
//| Comments                                                         |
//| 1.added time period under inputs in property sheet               |
//| 2.added defaults for line styles -outer solid, middle dash &     |
//|   inner dotted.                                                  |
//| 3.changed fib values to more precise numbers                     |
//| 4.changed SMAs to EMAs and price from MEDIAN to CLOSE            |
//| 5.parameterised blue & yellow lines - again check Inputs         |
//|                                                                  |
//|------------------------------------------------------------------|
//|                                                                  |
//| 1.0.2   11.12.05 isotonic                                        |
//| Comments                                                         |
//| 1.revised the signal alerts: select:                             |
//|   0 for dialog box; 1 for email; 2 for sound and 3 for speech    |
//|                                                                  |
//| 2.moving avg type parameterised - select number:                 |
//|   Constant Value Description                                     |
//|   MODE_SMA 0 Simple moving average,                              |
//|   MODE_EMA 1 Exponential moving average,                         |
//|   MODE_SMMA 2 Smoothed moving average,                           |
//|   MODE_LWMA 3 Linear weighted moving average.                    |
//|                                                                  |
//| 3.set timeframe by specifying in hourly units:                   |
//|   e.g. 0.25(15m), 0.5(30m), 1,4,24,168 etc                       |
//|                                                                  |
//| 4.applied price settings                                         |
//|   Constant Value Description                                     |
//|   PRICE_CLOSE 0 Close price.                                     |
//|   PRICE_OPEN 1 Open price.                                       |
//|   PRICE_HIGH 2 High price.                                       |
//|   PRICE_LOW 3 Low price.                                         |
//|   PRICE_MEDIAN 4 Median price, (high+low)/2.                     |
//|   PRICE_TYPICAL 5 Typical price, (high+low+close)/3.             |
//|   PRICE_WEIGHTED 6 Weighted close price, (high+low+close+close)/4| 
//|                                                                  |
//|------------------------------------------------------------------|
//|                                                                  |
//| 1.0.3   11.13.05 isotonic                                        |
//| Comments                                                         |
//| 1.revised the signal alerts: select:                             |
//|   0 for off; 1 for dialog box; 2 for email; 3 for sound and      | 
//!   4 for speech                                                   |
//|                                                                  |
//! 2. deviations allowed as doubles - inner < middle < outer        |
//|                                                                  |
//| 3. input parameters checked and set to default tops for bad values|
//+------------------------------------------------------------------+

//---- constants
#define   ALERT_OFF     0
#define   ALERT_DIALOG  1
#define   ALERT_EMAIL   2
#define   ALERT_SOUND   3
#define   ALERT_SPEECH  4 

//---- original author & copyright
#property copyright "dsvt1"
#property link      "dsvt1@yahoo.com"

//---- property sheet indicators 
#property indicator_separate_window
#property indicator_buffers 8
#property indicator_color1 Red
#property indicator_color2 Green
#property indicator_color3 Red
#property indicator_color4 Green
#property indicator_color5 Red
#property indicator_color6 Green
#property indicator_color7 Blue
#property indicator_color8 Yellow

//---- input parameters
//---- time period
extern int     maPeriod = 1;  // set in hourly units

//---- blue line period
extern int     maBlue = 150;  

//---- yellow line period
extern int     maYellow = 2;  

//---- ma type
extern int     maType = MODE_SMA;  

//---- applied prices
extern int     AppliedPrice = PRICE_CLOSE;

//---- signal alert 
extern int     BuySignal = ALERT_OFF;
extern int     SellSignal = ALERT_OFF;

//---- deviation levels
extern double  DevInner =  236;
extern double  DevMiddle = 382;
extern double  DevOuter =  618;

double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double ExtMapBuffer4[];
double ExtMapBuffer5[];
double ExtMapBuffer6[];
double ExtMapBuffer7[];
double ExtMapBuffer8[];

//+------------------------------------------------------------------+
//|Update for V4 - removed 50%R,added MA filter and median ref line  |
//+------------------------------------------------------------------+
int init()
  {
//---- check inputs and assume defaults for bad values
//---- 1. period 

   if (maPeriod < 0 || maPeriod > 168)
   {
      maPeriod = 1;  //tops 1 setting
   }
         
//---- 2. blue line
   if (maBlue < 0)
   {
      maBlue = 150;  //tops 1 setting
   }
   
//---- 3. yellow line 
   if (maYellow < 0)
   {
      maYellow = 2;    //tops 1 setting
   }
   if (maYellow > maBlue)
   {
      maYellow = maBlue;    
   }

//---- 4. ma setting
   if (maType < MODE_SMA || maType > MODE_LWMA)
   {
      maType = MODE_SMA;    //default to SMA
   }

//---- 5. applied price 
   if (AppliedPrice < PRICE_CLOSE || AppliedPrice > PRICE_WEIGHTED)
   {
      AppliedPrice = PRICE_CLOSE;    //default to PRICE_CLOSE
   }

//---- 6. buy signal
   if (BuySignal < ALERT_OFF || BuySignal > ALERT_SPEECH)
   {
      BuySignal = ALERT_OFF;    //default to ALERT_OFF
   }

//---- 7. sell signal 
   if (SellSignal < ALERT_OFF || SellSignal > ALERT_SPEECH)
   {
      SellSignal = ALERT_OFF;    //default to ALERT_OFF
   }

//---- 8. deviation values 

   if (DevInner < 0)
   {
      DevInner = 236;    
   }
   if (DevMiddle < 0)
   {
      DevMiddle = 382;    
   }
   if (DevOuter < 0)
   {
      DevOuter = 618;    
   }
   if (DevInner > DevMiddle)
   {
      DevInner = 236;    
      DevMiddle = 382;    
   }
   if (DevMiddle > DevOuter)
   {
      DevMiddle = 382;    
      DevOuter = 618;    
   }

//---- indicators
   SetIndexStyle(0,DRAW_LINE,STYLE_DOT);
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexStyle(1,DRAW_LINE,STYLE_DOT);
   SetIndexBuffer(1,ExtMapBuffer2);
   SetIndexStyle(2,DRAW_LINE,STYLE_DASH);
   SetIndexBuffer(2,ExtMapBuffer3);
   SetIndexStyle(3,DRAW_LINE,STYLE_DASH);
   SetIndexBuffer(3,ExtMapBuffer4);
   SetIndexStyle(4,DRAW_LINE,STYLE_SOLID);
   SetIndexBuffer(4,ExtMapBuffer5);
   SetIndexStyle(5,DRAW_LINE,STYLE_SOLID);
   SetIndexBuffer(5,ExtMapBuffer6);
   SetIndexStyle(6,DRAW_LINE,STYLE_SOLID);
   SetIndexBuffer(6,ExtMapBuffer7);
   SetIndexStyle(7,DRAW_LINE,STYLE_SOLID);
   SetIndexBuffer(7,ExtMapBuffer8);
 
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
  
  //---- returns bars count that does not changed after last indicator launch. In most cases same count of index values do not need for recalculation. Used for optimizing calculations. 
   int limit;
   int    counted_bars=IndicatorCounted();
  //---- check for possible errors
   if(counted_bars<0) return(-1);
  //---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;

  //---- local variables
  bool bBuy;
  bool bSell;
  int iPeriod;
  double dblDevInner;
  double dblDevMiddle;
  double dblDevOuter;

  //---- initialise locals
  iPeriod = 24 / maPeriod;
  dblDevInner = DevInner / 1000;
  dblDevMiddle = DevMiddle / 1000;
  dblDevOuter = DevOuter / 1000;

  //---- main loop
  for(int i=0; i<limit; i++)
    {
     //----shift set to 0 because SetIndexShift called abowe
      //---- changing deviation magic numbers to variables not working...why!?
      ExtMapBuffer1[i]=iEnvelopes(NULL, 0, iPeriod,maType,0,AppliedPrice,dblDevInner,MODE_UPPER,i);
      ExtMapBuffer2[i]=iEnvelopes(NULL, 0, iPeriod,maType,0,AppliedPrice,dblDevInner,MODE_LOWER,i);
      ExtMapBuffer3[i]=iEnvelopes(NULL, 0, iPeriod,maType,0,AppliedPrice,dblDevMiddle,MODE_UPPER,i);
      ExtMapBuffer4[i]=iEnvelopes(NULL, 0, iPeriod,maType,0,AppliedPrice,dblDevMiddle,MODE_LOWER,i);
      ExtMapBuffer5[i]=iEnvelopes(NULL, 0, iPeriod,maType,0,AppliedPrice,dblDevOuter,MODE_UPPER,i);
      ExtMapBuffer6[i]=iEnvelopes(NULL, 0, iPeriod,maType,0,AppliedPrice,dblDevOuter,MODE_LOWER,i);
      ExtMapBuffer7[i]=iMA(NULL,0,maBlue,0,maType,AppliedPrice,i);
      ExtMapBuffer8[i]=iMA(NULL,0,maYellow,0,maType,AppliedPrice,i);
    }
      
   //---- evalulate signal conditions 
   //---- 1.0.2 these signal alerts are untested so beware!

   bBuy = Close[i]==ExtMapBuffer2[i] || Close[i]==ExtMapBuffer4[i];
   bSell = Close[i]==ExtMapBuffer1[i] || Close[i]==ExtMapBuffer3[i];
   
   if (bBuy) 
   {
      SignalAlert ("Buy", BuySignal);
   }

   if (bSell)
   {
      SignalAlert ("Sell", SellSignal);
   }
            
//---- done  
   return(0);
  }


//+------------------------------------------------------------------+
//| Signal alert function                                            |
//| parameters:                                                      |
//| AlertType same as BuySignal or SellSignal above                  |
//| BuySell is either "Buy" or "Sell" string literal               |
//+------------------------------------------------------------------+

int SignalAlert (string BuySell, int AlertType)
  {

   string sMsg; 
   
//---- check inputs
   if (AlertType < ALERT_OFF || AlertType > ALERT_SPEECH)
   {
      AlertType = ALERT_OFF;    //default to ALERT_OFF
   }   

   sMsg = BuySell + " Signal triggered for "+Symbol()+" on the "+Period()+" minute chart.";
   
   switch (AlertType)
      {
         case ALERT_DIALOG:
            Alert(sMsg);
            break;
            
         case ALERT_EMAIL:
            SendMail(BuySell + " alert", sMsg);                
            break;
            
         case ALERT_SOUND:
            PlaySound("alert2.wav");
            break;
            
         case ALERT_SPEECH:
            SpeechText(sMsg);
            break;
      }
   
//----
   return(0);

  }

